@@ -62,6 +62,19 @@ alloc_uinput_device(const char *name)
 	return uinput_dev;
 }
 
+static inline int
+set_abs(const struct libevdev *dev, int fd, unsigned int code)
+{
+	const struct input_absinfo *abs = libevdev_get_abs_info(dev, code);
+	struct uinput_abs_setup abs_setup = {0};
+	int rc;
+
+	abs_setup.code = code;
+	abs_setup.absinfo = *abs;
+	rc = ioctl(fd, UI_ABS_SETUP, &abs_setup);
+	return rc;
+}
+
 static int
 set_evbits(const struct libevdev *dev, int fd, struct uinput_user_dev *uidev)
 {
@@ -113,13 +126,21 @@ set_evbits(const struct libevdev *dev, int fd, struct uinput_user_dev *uidev)
 				goto out;
 
 			if (type == EV_ABS) {
-				const struct input_absinfo *abs = libevdev_get_abs_info(dev, code);
-				uidev->absmin[code] = abs->minimum;
-				uidev->absmax[code] = abs->maximum;
-				uidev->absfuzz[code] = abs->fuzz;
-				uidev->absflat[code] = abs->flat;
-				/* uinput has no resolution in the device struct, this needs
-				 * to be fixed in the kernel */
+				if (uidev == NULL) {
+					rc = set_abs(dev, fd, code);
+					if (rc != 0)
+						goto out;
+				} else {
+					const struct input_absinfo *abs =
+						libevdev_get_abs_info(dev, code);
+
+					uidev->absmin[code] = abs->minimum;
+					uidev->absmax[code] = abs->maximum;
+					uidev->absfuzz[code] = abs->fuzz;
+					uidev->absflat[code] = abs->flat;
+					/* uinput has no resolution in the
+					 * device struct */
+				}
 			}
 		}
 
@@ -130,7 +151,7 @@ out:
 }
 
 static int
-set_props(const struct libevdev *dev, int fd, struct uinput_user_dev *uidev)
+set_props(const struct libevdev *dev, int fd)
 {
 	unsigned int prop;
 	int rc = 0;
@@ -271,13 +292,74 @@ fetch_syspath_and_devnode(struct libevdev_uinput *uinput_dev)
 	return uinput_dev->devnode ? 0 : -1;
 }
 
+static int
+uinput_create_write(const struct libevdev *dev, int fd,
+		    struct libevdev_uinput *new_device)
+{
+	int rc;
+	struct uinput_user_dev uidev;
+
+	memset(&uidev, 0, sizeof(uidev));
+
+	strncpy(uidev.name, libevdev_get_name(dev), UINPUT_MAX_NAME_SIZE - 1);
+	uidev.id.vendor = libevdev_get_id_vendor(dev);
+	uidev.id.product = libevdev_get_id_product(dev);
+	uidev.id.bustype = libevdev_get_id_bustype(dev);
+	uidev.id.version = libevdev_get_id_version(dev);
+
+	if (set_evbits(dev, fd, &uidev) != 0)
+		goto error;
+	if (set_props(dev, fd) != 0)
+		goto error;
+
+	rc = write(fd, &uidev, sizeof(uidev));
+	if (rc < 0)
+		goto error;
+	else if ((size_t)rc < sizeof(uidev)) {
+		errno = EINVAL;
+		goto error;
+	}
+
+	errno = 0;
+
+error:
+	return -errno;
+}
+
+static int
+uinput_create_DEV_SETUP(const struct libevdev *dev, int fd,
+			struct libevdev_uinput *new_device)
+{
+	int rc;
+	struct uinput_setup setup;
+
+	if (set_evbits(dev, fd, NULL) != 0)
+		goto error;
+	if (set_props(dev, fd) != 0)
+		goto error;
+
+	memset(&setup, 0, sizeof(setup));
+	strncpy(setup.name, libevdev_get_name(dev), UINPUT_MAX_NAME_SIZE - 1);
+	setup.id.vendor = libevdev_get_id_vendor(dev);
+	setup.id.product = libevdev_get_id_product(dev);
+	setup.id.bustype = libevdev_get_id_bustype(dev);
+	setup.id.version = libevdev_get_id_version(dev);
+	setup.ff_effects_max = libevdev_has_event_type(dev, EV_FF) ? 10 : 0;
+
+	rc = ioctl(fd, UI_DEV_SETUP, &setup);
+	if (rc == 0)
+		errno = 0;
+error:
+	return -errno;
+}
+
 LIBEVDEV_EXPORT int
 libevdev_uinput_create_from_device(const struct libevdev *dev, int fd, struct libevdev_uinput** uinput_dev)
 {
 	int rc;
-	struct uinput_user_dev uidev;
 	struct libevdev_uinput *new_device;
 	int close_fd_on_error = (fd == LIBEVDEV_UINPUT_OPEN_MANAGED);
+	unsigned int uinput_version = 0;
 
 	new_device = alloc_uinput_device(libevdev_get_name(dev));
 	if (!new_device)
@@ -295,26 +377,14 @@ libevdev_uinput_create_from_device(const struct libevdev *dev, int fd, struct li
 		goto error;
 	}
 
-	memset(&uidev, 0, sizeof(uidev));
+	if (ioctl(fd, UI_GET_VERSION, &uinput_version) == 0 &&
+	    uinput_version >= 5)
+		rc = uinput_create_DEV_SETUP(dev, fd, new_device);
+	else
+		rc = uinput_create_write(dev, fd, new_device);
 
-	strncpy(uidev.name, libevdev_get_name(dev), UINPUT_MAX_NAME_SIZE - 1);
-	uidev.id.vendor = libevdev_get_id_vendor(dev);
-	uidev.id.product = libevdev_get_id_product(dev);
-	uidev.id.bustype = libevdev_get_id_bustype(dev);
-	uidev.id.version = libevdev_get_id_version(dev);
-
-	if (set_evbits(dev, fd, &uidev) != 0)
+	if (rc != 0)
 		goto error;
-	if (set_props(dev, fd, &uidev) != 0)
-		goto error;
-
-	rc = write(fd, &uidev, sizeof(uidev));
-	if (rc < 0)
-		goto error;
-	else if ((size_t)rc < sizeof(uidev)) {
-		errno = EINVAL;
-		goto error;
-	}
 
 	/* ctime notes time before/after ioctl to help us filter out devices
 	   when traversing /sys/devices/virtual/input to find the device
